package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.ETC_MODBUS_TYPE)
public class EtcPersistenceHandler extends AbstractPersistenceHandler {

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_ETC.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_ETC.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, EtcPersistenceHandler.this.snowflakeConfig.snowflakeId());
                EtcPersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                EtcPersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 5, "solenoidValveStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 6, "controllerCurrentStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 7, "currentStatusRemainingTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 8, "currentStatusStartTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 9, "casPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 10, "oilPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 11, "plungerRiseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 12, "arrivalSensorDelayTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 13, "wellShutInTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 14, "afterFlowTime", Types.VARCHAR);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }
}
